@@ -34,13 +34,20 @@ class TrainingWrapper(nn.Module):
         self.net = Autopadder(net)
         self.max_seq_len = net.max_seq_len
 
-    def generate(self, start_tokens, seq_len, max_seq_len = None, eos_token = None, temperature = 1., filter_logits_fn = top_k, filter_thres = 0.9, **kwargs):
+    def generate(self, start_tokens, seq_len, eos_token = None, temperature = 1., filter_logits_fn = top_k, filter_thres = 0.9, **kwargs):
+        num_dims = len(start_tokens.shape)
+
+        if num_dims == 1:
+            start_tokens = start_tokens[None, :]
+
+        b, t = start_tokens.shape
+
         self.net.eval()
         out = start_tokens
 
         for _ in range(seq_len):
             x = out[:, -self.max_seq_len:]
-            logits = self.net(x)[:, -1, :]
+            logits = self.net(x, **kwargs)[:, -1, :]
             filtered_logits = filter_logits_fn(logits, thres = filter_thres)
             probs = F.softmax(filtered_logits / temperature, dim=-1)
             sample = torch.multinomial(probs, 1)
@@ -48,6 +55,11 @@ class TrainingWrapper(nn.Module):
             if eos_token is not None and (sample == eos_token).all():
                 break
 
+        out = out[:, t:]
+
+        if num_dims == 1:
+            out = out.squeeze(0)
+
         return out
 
     def forward(self, x, return_loss = False, **kwargs):
